//////////////////////////////////////////////////////////////////////////////////////////
//
// System iNtrusion Analysis and Reporting Environment (SNARE) - Audit Module
//
// This is the core of the SNARE audit subsystem. The audit module intercepts
// system calls, and saves off details associated with the call to a circular memory
// buffer, which is then read by the audit daemon.
//
///////////////////////////////////////////////////////////////////////////////////////////
// Copyright 1999-2001 InterSect Alliance Pty Ltd - http://www.intersectalliance.com/
///////////////////////////////////////////////////////////////////////////////////////////

// Includes
#include <linux/module.h>

#include <linux/reboot.h>
#include <linux/proc_fs.h>

#include <linux/net.h>
#include <linux/in.h>

#include <asm/uaccess.h>

#include <sys/syscall.h>

#include "audit.h"
#include "auditmodule.h"

// Global static variables
static void *read_position = (void *)NULL; // How far have we read through the node at the head of the list?

// /proc/auditinfo read position.
static char *	info_read_position;		// read position in the information proc entry.

static spinlock_t			audit_spin_lock;			// kernel locking
static struct task_struct *	auditdaemon_task_struct;	// Daemon interaction
unsigned long int flags; 								// CPU flags when using spin_lock_irqsave

extern void *sys_call_table[];

static int AUDIT_IS_RUNNING=0;	// Running flag

// For the /proc/audit device
static short audit_device_open=0;
static short audit_info_open=0;

// How many packets have we lost this session due to low buffer size?
static int lost_events=0;
static int total_events=0;

// Create an array to store the active events
static short active_audit[MAXAUDIT];

struct proc_dir_entry *ProcEntry;
struct proc_dir_entry *ProcInfoEntry;

// Kernel 2.4 and above only
static struct file_operations file_ops =
{
	read:	auditmodule_read,		// read
	ioctl:	auditmodule_ioctl,		// ioctl
	open:	auditmodule_open,		// open
	release:	auditmodule_close,	// release
};

static struct file_operations info_ops =
{
	read:	info_read,				// read
};

MODULE_AUTHOR("InterSect Alliance - www.intersectalliance.com");
MODULE_DESCRIPTION("Linux Audit Module");
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,14)
MODULE_LICENSE("GPL");
#endif

// Parameters for event selection.
const int file_class;			MODULE_PARM(file_class,"i");
MODULE_PARM_DESC(file_class,"File-related audit events");
int a_open;				MODULE_PARM(a_open,"i");
int a_creat;			MODULE_PARM(a_creat,"i");
int a_mkdir;			MODULE_PARM(a_mkdir,"i");
int a_unlink;			MODULE_PARM(a_unlink,"i");
int a_mknod;			MODULE_PARM(a_mknod,"i");
int a_rmdir;			MODULE_PARM(a_rmdir,"i");
int a_symlink;			MODULE_PARM(a_symlink,"i");
int a_link;				MODULE_PARM(a_link,"i");
int a_rename;			MODULE_PARM(a_rename,"i");
int a_truncate;			MODULE_PARM(a_truncate,"i");
//const int a_ftruncate64;	MODULE_PARM(a_ftruncate64,"i");
int a_truncate64;		MODULE_PARM(a_truncate64,"i");
//const int a_ftruncate;		MODULE_PARM(a_ftruncate,"i");

const int file_control_class;	MODULE_PARM(file_control_class,"i");
MODULE_PARM_DESC(file_control_class,"File control related audit events");
int a_chown;			MODULE_PARM(a_chown,"i");
int a_lchown;			MODULE_PARM(a_lchown,"i");
int a_chown32;			MODULE_PARM(a_chown32,"i");
int a_lchown32; 		MODULE_PARM(a_lchown32,"i");
int a_chmod; 			MODULE_PARM(a_chmod,"i");
//const int a_fchmod; 		MODULE_PARM(a_fchmod,"i");
//const int a_fchown;			MODULE_PARM(a_fchown,"i");

const int execute_class;		MODULE_PARM(execute_class,"i");
MODULE_PARM_DESC(execute_class,"Execute related audit events");
int a_execve;			MODULE_PARM(a_execve,"i");
int a_setuid; 			MODULE_PARM(a_setuid,"i");
int a_setreuid; 		MODULE_PARM(a_setreuid,"i");
int a_setresuid; 		MODULE_PARM(a_setresuid,"i");
int a_setuid32; 		MODULE_PARM(a_setuid32,"i");
int a_setreuid32; 		MODULE_PARM(a_setreuid32,"i");
int a_setresuid32;	 	MODULE_PARM(a_setresuid32,"i");
int a_setgid; 			MODULE_PARM(a_setgid,"i");
int a_setregid; 		MODULE_PARM(a_setregid,"i");
int a_setresgid;		MODULE_PARM(a_setresgid,"i");
int a_setgid32;			MODULE_PARM(a_setgid32,"i");
int a_setregid32;	 	MODULE_PARM(a_setregid32,"i");
int a_setresgid32;		MODULE_PARM(a_setresgid32,"i");
int a_exit;				MODULE_PARM(a_exit,"i");

const int admin_class;		MODULE_PARM(admin_class,"i");
MODULE_PARM_DESC(admin_class,"Administrative audit events");
int a_reboot;			MODULE_PARM(a_reboot,"i");
int a_chroot;			MODULE_PARM(a_chroot,"i");
int a_create_module;	MODULE_PARM(a_create_module,"i");
//int a_delete_module;	MODULE_PARM(a_delete_module,"i");

const int network_class;	MODULE_PARM(network_class,"i");
int a_socketcall;		MODULE_PARM(a_socketcall,"i");

int all;				MODULE_PARM(all,"i");
MODULE_PARM_DESC(all,"Turn on all audit events.");



///////////////////////////////////////////////////////////////////////////////
// Linked List
struct _node
{
	void * location;	// Location in RAM of the allocated chunk
	int size;			// Size of the chunk
	struct _node *next;	// Next node in the series.
};

typedef struct _node Node;

Node * list_head=(Node *)NULL;
Node * list_tail=(Node *)NULL;

int CURRENTBUFFER=0;




////////////////////////////////////////////////////////////////////////////////
// Initialise the module
// Set up our event overlay functions.
////////////////////////////////////////////////////////////////////////////////
int init_module(void)
{
	long newbuffersize=0;
	int count=0;
	
	AUDIT_IS_RUNNING = FALSE;
	auditdaemon_task_struct=NULL;

	// Initialise our active event storage array.
	for(count=0;count <= MAXAUDIT; count++) {
		active_audit[count]=0;
	}

	// Kernel lock
	audit_spin_lock = SPIN_LOCK_UNLOCKED;
	
	// From here on in, the read routine could be signaled.
	// Therefore, wherever we write to the audit buffer, or we rely on
	// any of the variables that the read routine changes,
	// we should lock the kernel first.

	// Would do this using proc_register in kernel 2.2 - see http://linuxkernel.to//port-2.4/eng/lkp-all.html
	if ((ProcEntry = create_proc_entry(AUDITDEV_NAME, S_IRUSR | S_IWUSR, NULL)) == NULL)
		return(1);
		
	ProcEntry->proc_fops = &file_ops;

	if ((ProcInfoEntry = create_proc_entry(AUDITINFO_NAME, S_IRUGO | S_IWUSR, NULL)) == NULL)
		return(1);
		
	ProcInfoEntry->proc_fops = &info_ops;


	// Now: Verify what audit events the user wants, and insert ourselves into the
	// relevant system calls.

	printk("SNARE Audit Module: Version %d.%d%d initialized\n", AUDITMODULE_MAJOR_VERSION,
			AUDITMODULE_MINOR_VERSION, AUDITMODULE_PATCH_VERSION);

	// Set up auditable functions
	if(a_open || all || file_class)					audit_on(SYS_open);
	if(a_creat || all || file_class)				audit_on(SYS_creat);
	if(a_execve || all || execute_class)			audit_on(SYS_execve);
	if(a_exit || all || execute_class)				audit_on(SYS_exit);
	if(a_mkdir || all || file_class)				audit_on(SYS_mkdir);
	if(a_unlink || all || file_class)				audit_on(SYS_unlink);
	if(a_mknod || all || file_class)				audit_on(SYS_mknod);
	if(a_rmdir || all || file_class)				audit_on(SYS_rmdir);
	if(a_chown || all || file_control_class)		audit_on(SYS_chown);
	if(a_lchown || all || file_control_class)		audit_on(SYS_lchown);
	if(a_chown32 || all || file_control_class)		audit_on(SYS_chown32);
	if(a_lchown32 || all || file_control_class)		audit_on(SYS_lchown32);
	if(a_chmod || all || file_control_class)		audit_on(SYS_chmod);
	if(a_symlink || all || file_class)				audit_on(SYS_symlink);
	if(a_link || all || file_class)					audit_on(SYS_link);
	if(a_rename || all || file_class)				audit_on(SYS_rename);
	
	if(a_reboot || all || admin_class)				audit_on(SYS_reboot);
	if(a_truncate || all || file_class)				audit_on(SYS_truncate);
	if(a_chroot || all || admin_class)				audit_on(SYS_chroot);

	if(a_create_module || all || admin_class)		audit_on(SYS_create_module);
	//if(a_delete_module || all || admin_class)		audit_on(SYS_delete_module);

	if(a_setuid || all || execute_class)			audit_on(SYS_setuid);
	if(a_setreuid || all || execute_class)			audit_on(SYS_setreuid);
	if(a_setresuid || all || execute_class)			audit_on(SYS_setresuid);
	if(a_setuid32 || all || execute_class)			audit_on(SYS_setuid32);
	if(a_setreuid32 || all || execute_class)		audit_on(SYS_setreuid32);
	if(a_setresuid32 || all || execute_class)		audit_on(SYS_setresuid32);
	if(a_setgid || all || execute_class)			audit_on(SYS_setgid);
	if(a_setregid || all || execute_class)			audit_on(SYS_setregid);
	if(a_setresgid || all || execute_class)			audit_on(SYS_setresgid);
	if(a_setgid32 || all || execute_class)			audit_on(SYS_setgid32);
	if(a_setregid32 || all || execute_class)		audit_on(SYS_setregid32);
	if(a_setresgid32 || all || execute_class)		audit_on(SYS_setresgid32);
	// if(a_ftruncate || all || file_class)			audit_on(SYS_ftruncate);	// Not required - truncate covers this.
	// if(a_fchmod || all || file_control_class)		audit_on(SYS_fchmod);
	// if(a_ftruncate64 || all || file_class)			audit_on(SYS_ftruncate64);
	if(a_truncate64 || all || file_class)			audit_on(SYS_truncate64);
	// if(a_fchown || all || file_control_class)		audit_on(SYS_fchown);

	if(a_socketcall || all || network_class)		audit_on(SYS_socketcall);
	
	return 0;
}

////////////////////////////////////////////////////////////////////////////////
// This routine handles ioctl messages from the audit daemon.
////////////////////////////////////////////////////////////////////////////////

int auditmodule_ioctl(struct inode* node, struct file*  the_file, unsigned int  command, unsigned long arg)
{
	static int counter=0;

	// If the audit module is running, there is not much point starting again!
	if((AUDIT_IS_RUNNING == TRUE) && (command == AUDIT_START))
		return -EBUSY;

	if(command == AUDIT_START) {
		AUDIT_IS_RUNNING = TRUE;
	} else if(command == AUDIT_STOP) {
		AUDIT_IS_RUNNING = FALSE;
	} else if(command == AUDIT_LOSTEVENTS) {
		return(lost_events);
	} else if(command == AUDIT_TOTALEVENTS) {
		return(total_events);
	} else if(command == AUDIT_INFO) {
		// This is designed (with the assistance of the static int at the top of this routine)
		// to return one system call number per AUDIT_EVENTLIST ioctl.
		// the audit daemon code will look something like this:
		// while(event=ioctl(/dev/audit AUDIT_EVENTLIST)) { print the name of the event }
		// If we've hit the end of the array, reset our counter, and return 'end of array' (0)
		//
		// Note that we don't use this capability within auditd at the moment.
		//
		counter++;

		while(active_audit[counter]==0 && counter <= MAXAUDIT) {
			counter++;
		}
		
		if(counter <= MAXAUDIT) {
			return(active_audit[counter]);
		} else {
			counter=0;
			return(0);
		}
	} else if (command == AUDIT_EVENT_ON) {
		audit_on(arg);
	} else if (command == AUDIT_EVENT_OFF) {
		audit_off(arg);
		// Eg: ioctl(x,SYS_exit);
	} else if (command == AUDIT_FLUSH) {
			int counter;
			for(counter=1; counter<=MAXAUDIT; counter++) {
				if(active_audit[counter]) audit_off(active_audit[counter]);
			}		

			// Also reset the lost/total event counters here.
			lost_events=0;
			total_events=0;
	} else {
		return -ENOSYS;
	}

	return 0;
}


////////////////////////////////////////////////////////////////////////////////
// Here we give the user the data.
// NOTE that this module may be executed at any time - even in the middle
// of the execution of something else. Hence, any variables that are changed
// by this routine should be surrounded by a kernel lock if they are used
// elsewhere!
//
// In particular:
//  read_position
//  CURRENTBUFFER
//
// Let me just repeat this in another way:
// WARNING: DO NOT CHANGE OR RELY ON THE VARIABLES ABOVE UNLESS YOU ARE
//		  IN A SPIN_LOCKED STATE - THEY MAY BE CORRUPTED BY THE
//		  AUDIT_READ MODULE, WHICH IS CALLED BY SIGNAL/INTERRUPT
//
// Unfortunately, we also need to spin-lock within this routine also.
// Otherwise, an interrupt might occur when we were half way through modifying
// something like read_position - and if the interrupt happens to generate an
// audit event, then we may be copying data to the wrong location in the array,
// leading to data corruption.
// Spin locking too much code slows down the system unfortunately. I'll try
// and keep it minimal.
////////////////////////////////////////////////////////////////////////////////

static ssize_t auditmodule_read(struct file *file, char *filebuffer, size_t length, loff_t *ppos)
{
	// Number of bytes actually written to the buffer
	int bytes_read = 0;
	int bytes_avail_to_send=0;
	int bytes_to_write=0;

	spin_lock_irqsave(&audit_spin_lock, flags);
	
	// Do we have any data?
	if(list_head == (Node *)NULL) {
		spin_unlock_irqrestore(&audit_spin_lock, flags);
		return(0);
	}
	
	if(read_position==(Node *)NULL) {
		// Our first time into this routine, or first time back after all data has been read.
		read_position=list_head->location;
	}
	
	bytes_avail_to_send = (int)(((void *)list_head->location + list_head->size) - (void *)read_position);
	
	if(bytes_avail_to_send < length) {
		bytes_to_write=bytes_avail_to_send;
	} else {
		bytes_to_write=length;
	}
	
	if(copy_to_user(filebuffer,read_position,bytes_to_write))
	{
		spin_unlock_irqrestore(&audit_spin_lock, flags);
		return -EFAULT;
	}

	read_position += bytes_to_write;

	// Have we reached the end of the current nodes data?
	if((void *)read_position >= ((void *)list_head->location + list_head->size)) {
		Node *tempnode;

		tempnode = list_head;
		list_head = list_head->next;
		
		// Excellent, we're at the end of the current audit event.
		// Remove this from the linked list, and set the read_position to the next record.
		if(list_head != (Node *)NULL) {
			read_position = list_head->location;
		} else {
			read_position = (Node *)NULL;
		}

		// No events remaining? Set list_tail to null also.
		if(tempnode->next == (Node *)NULL) {
			list_tail=(Node *)NULL;
		}
	
		CURRENTBUFFER -= (tempnode->size + sizeof(Node));
	
		// NOTE: If you look at /proc/meminfo before and after these calls, the memory will not SEEM to be released.
		//       However, it's just that the kernel does not release the memory IMMEDIATELY. It seems to wait for
		//		 an appropriate opportunity before actually giving the RAM back to the system.
		kfree(tempnode->location);
		kfree(tempnode);
	}
	
	spin_unlock_irqrestore(&audit_spin_lock, flags);		

	return(bytes_to_write);
}


////////////////////////////////////////////////////////////////////////////////
// open the device
////////////////////////////////////////////////////////////////////////////////
static int auditmodule_open(struct inode* node, struct file*  the_file)
{
	int device_minor = MINOR(node->i_rdev) & 0xf;  // Device minor number

	// reset the lost packets indicator
	lost_events=0;
	total_events=0;

	// Can only have one audit device, and it can only be open once
	if(device_minor)
		return -ENODEV;
	if(audit_device_open)
		return -EBUSY;

	// Ok, we're open.
	audit_device_open=1;

	// Fetch the task structure of the process that opened the device
	auditdaemon_task_struct = current;

	printk("Auditing: Process %d has opened the device\n", auditdaemon_task_struct->pid);

	MOD_INC_USE_COUNT; // Increment module count

	return 0;
}

////////////////////////////////////////////////////////////////////////////////
// close the device and release resources
////////////////////////////////////////////////////////////////////////////////
static int auditmodule_close(struct inode* node, struct file*  the_file)
{
	Node *currentnode;
	Node *tempnode;
	
	auditdaemon_task_struct = NULL;
	AUDIT_IS_RUNNING = FALSE;
	audit_device_open=0;
	auditdaemon_task_struct=NULL;

	printk("AUDITMODULE: Audit daemon has closed dev/audit.\n");
	printk("AUDITMODULE: Events lost due low memory this session: %d\n",lost_events);
	printk("AUDITMODULE: Total Events processed this session: %d\n",total_events);
	lost_events=0;
	total_events=0;
	
	// Reset all our buffers here.
	// con: reset buffers may lose us some events.
	// pro: we could have been terminated half way through reading something, and auditd will not recover.
	spin_lock_irqsave(&audit_spin_lock, flags);

	// Free our linked list, just in case there are any left over.
	currentnode=list_head;
	while(currentnode != (Node *)NULL) {
		tempnode=currentnode;
		currentnode=currentnode->next;
		kfree(tempnode->location);
		kfree(tempnode);
	}
	
	//	write_position=read_position=storage_buffer_start;
	spin_unlock_irqrestore(&audit_spin_lock, flags);

	MOD_DEC_USE_COUNT; // Decrement module usage
	
	return 0;
}

// /proc/auditinfo open.
static int info_open(struct inode* node, struct file*  the_file)
{
	int device_minor = MINOR(node->i_rdev) & 0xf;  /* Device minor number */

	// Only one open at a time thanks.
	if(device_minor)
		return -ENODEV;
	if(audit_info_open)
		return -EBUSY;

	// Ok, we're open.
	audit_info_open=1;
	info_read_position=(char *)NULL;

	return 0;
}

static int info_close(struct inode* node, struct file*  the_file)
{
	audit_info_open=0;
	info_read_position=(char *)NULL;

	return 0;
}

static ssize_t info_read(struct file *file, char *filebuffer, size_t length, loff_t *ppos)
{
	// Would prefer to dynamically malloc this, but we run the risk of not freeing at program termination.
	static char buffer[150]="";
	static int finished=0;
	int bytes_to_write=0;
	
	if(finished) {
		finished=0;
		// Return zero to indicate EOF
		return(0);
	}

	if(info_read_position == (char *)NULL) {
		char processid[20]="No Daemon Running";
				
		if(auditdaemon_task_struct) {
			if(auditdaemon_task_struct->pid && auditdaemon_task_struct->pid <= 65535) {
				// WOULD REALLY prefer snprintf here.
				sprintf(processid,"%d",(int)auditdaemon_task_struct->pid);
			}
		}
		
		// Would prefer to use snprintf here.
		// However, the size of these variables are under our control
		// (except pid, which is going to be <= maxint. (5 characters)
		// Therefore we need a buffer max of around 100 characters. Allow 150 just in case.
		// SNARE Version: 999.999.999XAudit Active: 9XAudit Process ID: 99999XEvents Lost This Session: 99999XX
		sprintf(buffer,"SNARE Version: %d.%d.%d\nAudit Active: %d\nAudit Process ID: %s\nEvents Lost This Session: %d\nEvents Processed This Session: %d\n",
			AUDITMODULE_MAJOR_VERSION,AUDITMODULE_MINOR_VERSION,AUDITMODULE_PATCH_VERSION,
			AUDIT_IS_RUNNING,
			processid, lost_events,total_events);
		
		info_read_position=buffer;
	}
			
	bytes_to_write=strlen(info_read_position);
	if(bytes_to_write > length) {
		if(copy_to_user(filebuffer,info_read_position,length))
		{
			return -EFAULT;
		}
		info_read_position+=length;
		return(length);
	} else {
		// Excellent The entire buffer has been completed.
		// Free the memory, return the pointer to zero.
		if(copy_to_user(filebuffer,info_read_position,bytes_to_write))
		{
			return -EFAULT;
		}
		info_read_position=(char *)NULL;
		finished=1;
		return(bytes_to_write);
	}

	return(0);
}

////////////////////////////////////////////////////////////////////////////////
// Signal the audit daemon that is reading from /dev/audit (if it is there).
// Note that this is called from within audit_event, but is not IRQ locked.
////////////////////////////////////////////////////////////////////////////////
int signal_auditd()
{
	static struct siginfo sig_info;	/* Signal information */

	// Not much use sending signals to a non-existant process.
	if(audit_device_open && AUDIT_IS_RUNNING)
	{
		// Setup signal information and send it
		sig_info.si_signo = SIGIO;
		sig_info.si_errno = 0;
		sig_info.si_code  = SI_KERNEL;
		send_sig_info(SIGIO, &sig_info, auditdaemon_task_struct);
		return(1);
	}
	return(0);
}


////////////////////////////////////////////////////////////////////////////////
// Free buffers, release our overlay functions and exit.
////////////////////////////////////////////////////////////////////////////////
void cleanup_module(void)
{
	int counter=0;
	// printk("Auditing: Cleaning up module\n");

	// Lock down the kernel, so no new events can occur.
	spin_lock_irqsave(&audit_spin_lock, flags);
	AUDIT_IS_RUNNING=0;

	// remove the proc entry, so the daemon won't try to get more data
	remove_proc_entry(AUDITDEV_NAME,NULL);
	remove_proc_entry(AUDITINFO_NAME,NULL);

	// Release the audited functions - skip AUDIT_NULL though.
	for(counter=1; counter<=MAXAUDIT; counter++) {
		if(active_audit[counter]) audit_off(active_audit[counter]);
	}

	spin_unlock_irqrestore(&audit_spin_lock, flags);

	printk("SNARE Audit Module: Exiting\n");
}

// Don't turn it on twice.
void audit_on(int auditnum)
{
	// printk("Debug: Turning on event %d\n",auditnum);
	switch (auditnum)
	{
		case SYS_open:
			if(active_audit[AUDIT_open]) return;
			orig_open=sys_call_table[SYS_open];
			sys_call_table[SYS_open]=audit_open;
			active_audit[AUDIT_open]=SYS_open;
			break;
		case SYS_creat:
			if(active_audit[AUDIT_creat]) return;
			orig_creat=sys_call_table[SYS_creat];
			sys_call_table[SYS_creat]=audit_creat;
			active_audit[AUDIT_creat]=SYS_creat;
			break;
		case SYS_execve:
			if(active_audit[AUDIT_execve]) return;
			orig_execve=sys_call_table[SYS_execve];
			sys_call_table[SYS_execve]=audit_execve;
			active_audit[AUDIT_execve]=SYS_execve;
			break;
		case SYS_exit:
			if(active_audit[AUDIT_exit]) return;
			orig_exit=sys_call_table[SYS_exit];
			sys_call_table[SYS_exit]=audit_exit;
			active_audit[AUDIT_exit]=SYS_exit;
			break;
		case SYS_mkdir:
			if(active_audit[AUDIT_mkdir]) return;
			orig_mkdir=sys_call_table[SYS_mkdir];
			sys_call_table[SYS_mkdir]=audit_mkdir;
			active_audit[AUDIT_mkdir]=SYS_mkdir;
			break;
		case SYS_unlink:
			if(active_audit[AUDIT_unlink]) return;
			orig_unlink=sys_call_table[SYS_unlink];
			sys_call_table[SYS_unlink]=audit_unlink;
			active_audit[AUDIT_unlink]=SYS_unlink;
			break;
		case SYS_mknod:
			if(active_audit[AUDIT_mknod]) return;
			orig_mknod=sys_call_table[SYS_mknod];
			sys_call_table[SYS_mknod]=audit_mknod;
			active_audit[AUDIT_mknod]=SYS_mknod;
			break;
		case SYS_rmdir:
			if(active_audit[AUDIT_rmdir]) return;
			orig_rmdir=sys_call_table[SYS_rmdir];
			sys_call_table[SYS_rmdir]=audit_rmdir;
			active_audit[AUDIT_rmdir]=SYS_rmdir;
			break;
		case SYS_chown:
			if(active_audit[AUDIT_chown]) return;
			orig_chown=sys_call_table[SYS_chown];
			sys_call_table[SYS_chown]=audit_chown;
			active_audit[AUDIT_chown]=SYS_chown;
			break;
		case SYS_lchown:
			if(active_audit[AUDIT_lchown]) return;
			orig_lchown=sys_call_table[SYS_lchown];
			sys_call_table[SYS_lchown]=audit_lchown;
			active_audit[AUDIT_lchown]=SYS_lchown;
			break;
		case SYS_chown32:
			if(active_audit[AUDIT_chown32]) return;
			orig_chown32=sys_call_table[SYS_chown32];
			sys_call_table[SYS_chown32]=audit_chown32;
			active_audit[AUDIT_chown32]=SYS_chown32;
			break;
		case SYS_lchown32:
			if(active_audit[AUDIT_lchown32]) return;
			orig_lchown32=sys_call_table[SYS_lchown32];
			sys_call_table[SYS_lchown32]=audit_lchown32;
			active_audit[AUDIT_lchown32]=SYS_lchown32;
			break;
		case SYS_chmod:
			if(active_audit[AUDIT_chmod]) return;
			orig_chmod=sys_call_table[SYS_chmod];
			sys_call_table[SYS_chmod]=audit_chmod;
			active_audit[AUDIT_chmod]=SYS_chmod;
			break;
		case SYS_symlink:
			if(active_audit[AUDIT_symlink]) return;
			orig_symlink=sys_call_table[SYS_symlink];
			sys_call_table[SYS_symlink]=audit_symlink;
			active_audit[AUDIT_symlink]=SYS_symlink;
			break;
		case SYS_link:
			if(active_audit[AUDIT_link]) return;
			orig_link=sys_call_table[SYS_link];
			sys_call_table[SYS_link]=audit_link;
			active_audit[AUDIT_link]=SYS_link;
			break;
		case SYS_rename:
			if(active_audit[AUDIT_rename]) return;
			orig_rename=sys_call_table[SYS_rename];
			sys_call_table[SYS_rename]=audit_rename;
			active_audit[AUDIT_rename]=SYS_rename;
			break;
		case SYS_reboot:
			if(active_audit[AUDIT_reboot]) return;
			orig_reboot=sys_call_table[SYS_reboot];
			sys_call_table[SYS_reboot]=audit_reboot;
			active_audit[AUDIT_reboot]=SYS_reboot;
			break;
		case SYS_truncate:
			if(active_audit[AUDIT_truncate]) return;
			orig_truncate=sys_call_table[SYS_truncate];
			sys_call_table[SYS_truncate]=audit_truncate;
			active_audit[AUDIT_truncate]=SYS_truncate;
			break;
		case SYS_chroot:
			if(active_audit[AUDIT_chroot]) return;
			orig_chroot=sys_call_table[SYS_chroot];
			sys_call_table[SYS_chroot]=audit_chroot;
			active_audit[AUDIT_chroot]=SYS_chroot;
			break;
		case SYS_setuid:
			if(active_audit[AUDIT_setuid]) return;
			orig_setuid=sys_call_table[SYS_setuid];
			sys_call_table[SYS_setuid]=audit_setuid;
			active_audit[AUDIT_setuid]=SYS_setuid;
			break;
		case SYS_setreuid:
			if(active_audit[AUDIT_setreuid]) return;
			orig_setreuid=sys_call_table[SYS_setreuid];
			sys_call_table[SYS_setreuid]=audit_setreuid;
			active_audit[AUDIT_setreuid]=SYS_setreuid;
			break;
		case SYS_setresuid:
			if(active_audit[AUDIT_setresuid]) return;
			orig_setresuid=sys_call_table[SYS_setresuid];
			sys_call_table[SYS_setresuid]=audit_setresuid;
			active_audit[AUDIT_setresuid]=SYS_setresuid;
			break;
		case SYS_setuid32:
			if(active_audit[AUDIT_setuid32]) return;
			orig_setuid32=sys_call_table[SYS_setuid32];
			sys_call_table[SYS_setuid32]=audit_setuid32;
			active_audit[AUDIT_setuid32]=SYS_setuid32;
			break;
		case SYS_setreuid32:
			if(active_audit[AUDIT_setreuid32]) return;
			orig_setreuid32=sys_call_table[SYS_setreuid32];
			sys_call_table[SYS_setreuid32]=audit_setreuid32;
			active_audit[AUDIT_setreuid32]=SYS_setreuid32;
			break;
		case SYS_setresuid32:
			if(active_audit[AUDIT_setresuid32]) return;
			orig_setresuid32=sys_call_table[SYS_setresuid32];
			sys_call_table[SYS_setresuid32]=audit_setresuid32;
			active_audit[AUDIT_setresuid32]=SYS_setresuid32;
			break;
		case SYS_setgid:
			if(active_audit[AUDIT_setgid]) return;
			orig_setgid=sys_call_table[SYS_setgid];
			sys_call_table[SYS_setgid]=audit_setgid;
			active_audit[AUDIT_setgid]=SYS_setgid;
			break;
		case SYS_setregid:
			if(active_audit[AUDIT_setregid]) return;
			orig_setregid=sys_call_table[SYS_setregid];
			sys_call_table[SYS_setregid]=audit_setregid;
			active_audit[AUDIT_setregid]=SYS_setregid;
			break;
		case SYS_setresgid:
			if(active_audit[AUDIT_setresgid]) return;
			orig_setresgid=sys_call_table[SYS_setresgid];
			sys_call_table[SYS_setresgid]=audit_setresgid;
			active_audit[AUDIT_setresgid]=SYS_setresgid;
			break;
		case SYS_setgid32:
			if(active_audit[AUDIT_setgid32]) return;
			orig_setgid32=sys_call_table[SYS_setgid32];
			sys_call_table[SYS_setgid32]=audit_setgid32;
			active_audit[AUDIT_setgid32]=SYS_setgid32;
			break;
		case SYS_setregid32:
			if(active_audit[AUDIT_setregid32]) return;
			orig_setregid32=sys_call_table[SYS_setregid32];
			sys_call_table[SYS_setregid32]=audit_setregid32;
			active_audit[AUDIT_setregid32]=SYS_setregid32;
			break;
		case SYS_setresgid32:
			if(active_audit[AUDIT_setresgid32]) return;
			orig_setresgid32=sys_call_table[SYS_setresgid32];
			sys_call_table[SYS_setresgid32]=audit_setresgid32;
			active_audit[AUDIT_setresgid32]=SYS_setresgid32;
			break;
		case SYS_truncate64:
			if(active_audit[AUDIT_truncate64]) return;
			orig_truncate64=sys_call_table[SYS_truncate64];
			sys_call_table[SYS_truncate64]=audit_truncate64;
			active_audit[AUDIT_truncate64]=SYS_truncate64;
			break;
		case SYS_socketcall:
			if(active_audit[AUDIT_socketcall]) return;
			orig_socketcall=sys_call_table[SYS_socketcall];
			sys_call_table[SYS_socketcall]=audit_socketcall;
			active_audit[AUDIT_socketcall]=SYS_socketcall;
			break;
		case SYS_create_module:
			if(active_audit[AUDIT_create_module]) return;
			orig_create_module=sys_call_table[SYS_create_module];
			sys_call_table[SYS_create_module]=audit_create_module;
			active_audit[AUDIT_create_module]=SYS_create_module;
			break;
		//case SYS_delete_module:
		//	if(active_audit[AUDIT_delete_module]) return;
		//	orig_delete_module=sys_call_table[SYS_delete_module];
		//	sys_call_table[SYS_delete_module]=audit_delete_module;
		//	active_audit[AUDIT_delete_module]=SYS_delete_module;
		//	break;
			
// No need for the calls below, they are covered by their non f* calls (eg: ftruncate calls truncate64 on my system).
//		case SYS_ftruncate:
//			if(active_audit[AUDIT_ftruncate]) return;
//			orig_ftruncate=sys_call_table[SYS_ftruncate];
//			sys_call_table[SYS_ftruncate]=audit_ftruncate;
//			active_audit[AUDIT_ftruncate]=SYS_ftruncate;
//			break;
//		case SYS_fchmod:
//			if(active_audit[AUDIT_fchmod]) return;
//			orig_fchmod=sys_call_table[SYS_fchmod];
//			sys_call_table[SYS_fchmod]=audit_fchmod;
//			active_audit[AUDIT_fchmod]=SYS_fchmod;
//			break;
//		case SYS_ftruncate64:
//			if(active_audit[AUDIT_ftruncate64]) return;
//			orig_ftruncate64=sys_call_table[SYS_ftruncate64];
//			sys_call_table[SYS_ftruncate64]=audit_ftruncate64;
//			active_audit[AUDIT_ftruncate64]=SYS_ftruncate64;
//			break;
//		case SYS_fchown:
//			if(active_audit[AUDIT_fchown]) return;
//			orig_fchown=sys_call_table[SYS_fchown];
//			sys_call_table[SYS_fchown]=audit_fchown;
//			active_audit[AUDIT_fchown]=SYS_fchown;
//			break;
	}
}

void audit_off(int auditnum)
{
	switch (auditnum)
	{
		case SYS_open:
			if(active_audit[AUDIT_open]==0) return;
			sys_call_table[SYS_open]=orig_open;
			active_audit[AUDIT_open]=0;
			break;
		case SYS_creat:
			if(active_audit[AUDIT_creat]==0) return;
			sys_call_table[SYS_creat]=orig_creat;
			active_audit[AUDIT_creat]=0;
			break;
		case SYS_execve:
			if(active_audit[AUDIT_execve]==0) return;
			sys_call_table[SYS_execve]=orig_execve;
			active_audit[AUDIT_execve]=0;
			break;
		case SYS_exit:
			if(active_audit[AUDIT_exit]==0) return;
			sys_call_table[SYS_exit]=orig_exit;
			active_audit[AUDIT_exit]=0;
			break;
		case SYS_mkdir:
			if(active_audit[AUDIT_mkdir]==0) return;
			sys_call_table[SYS_mkdir]=orig_mkdir;
			active_audit[AUDIT_mkdir]=0;
			break;
		case SYS_unlink:
			if(active_audit[AUDIT_unlink]==0) return;
			sys_call_table[SYS_unlink]=orig_unlink;
			active_audit[AUDIT_unlink]=0;
			break;
		case SYS_mknod:
			if(active_audit[AUDIT_mknod]==0) return;
			sys_call_table[SYS_mknod]=orig_mknod;
			active_audit[AUDIT_mknod]=0;
			break;
		case SYS_rmdir:
			if(active_audit[AUDIT_rmdir]==0) return;
			sys_call_table[SYS_rmdir]=orig_rmdir;
			active_audit[AUDIT_rmdir]=0;
			break;
		case SYS_chown:
			if(active_audit[AUDIT_chown]==0) return;
			sys_call_table[SYS_chown]=orig_chown;
			active_audit[AUDIT_chown]=0;
			break;
		case SYS_lchown:
			if(active_audit[AUDIT_lchown]==0) return;
			sys_call_table[SYS_lchown]=orig_lchown;
			active_audit[AUDIT_lchown]=0;
			break;
		case SYS_chown32:
			if(active_audit[AUDIT_chown32]==0) return;
			sys_call_table[SYS_chown32]=orig_chown32;
			active_audit[AUDIT_chown32]=0;
			break;
		case SYS_lchown32:
			if(active_audit[AUDIT_lchown32]==0) return;
			sys_call_table[SYS_lchown32]=orig_lchown32;
			active_audit[AUDIT_lchown32]=0;
			break;
		case SYS_chmod:
			if(active_audit[AUDIT_chmod]==0) return;
			sys_call_table[SYS_chmod]=orig_chmod;
			active_audit[AUDIT_chmod]=0;
			break;
		case SYS_symlink:
			if(active_audit[AUDIT_symlink]==0) return;
			sys_call_table[SYS_symlink]=orig_symlink;
			active_audit[AUDIT_symlink]=0;
			break;
		case SYS_link:
			if(active_audit[AUDIT_link]==0) return;
			sys_call_table[SYS_link]=orig_link;
			active_audit[AUDIT_link]=0;
			break;
		case SYS_rename:
			if(active_audit[AUDIT_rename]==0) return;
			sys_call_table[SYS_rename]=orig_rename;
			active_audit[AUDIT_rename]=0;
			break;
		case SYS_reboot:
			if(active_audit[AUDIT_reboot]==0) return;
			sys_call_table[SYS_reboot]=orig_reboot;
			active_audit[AUDIT_reboot]=0;
			break;
		case SYS_truncate:
			if(active_audit[AUDIT_truncate]==0) return;
			sys_call_table[SYS_truncate]=orig_truncate;
			active_audit[AUDIT_truncate]=0;
			break;
		case SYS_chroot:
			if(active_audit[AUDIT_chroot]==0) return;
			sys_call_table[SYS_chroot]=orig_chroot;
			active_audit[AUDIT_chroot]=0;
			break;
		case SYS_setuid:
			if(active_audit[AUDIT_setuid]==0) return;
			sys_call_table[SYS_setuid]=orig_setuid;
			active_audit[AUDIT_setuid]=0;
			break;
		case SYS_setreuid:
			if(active_audit[AUDIT_setreuid]==0) return;
			sys_call_table[SYS_setreuid]=orig_setreuid;
			active_audit[AUDIT_setreuid]=0;
			break;
		case SYS_setresuid:
			if(active_audit[AUDIT_setresuid]==0) return;
			sys_call_table[SYS_setresuid]=orig_setresuid;
			active_audit[AUDIT_setresuid]=0;
			break;
		case SYS_setuid32:
			if(active_audit[AUDIT_setuid32]==0) return;
			sys_call_table[SYS_setuid32]=orig_setuid32;
			active_audit[AUDIT_setuid32]=0;
			break;
		case SYS_setreuid32:
			if(active_audit[AUDIT_setreuid32]==0) return;
			sys_call_table[SYS_setreuid32]=orig_setreuid32;
			active_audit[AUDIT_setreuid32]=0;
			break;
		case SYS_setresuid32:
			if(active_audit[AUDIT_setresuid32]==0) return;
			sys_call_table[SYS_setresuid32]=orig_setresuid32;
			active_audit[AUDIT_setresuid32]=0;
			break;
		case SYS_setgid:
			if(active_audit[AUDIT_setgid]==0) return;
			sys_call_table[SYS_setgid]=orig_setgid;
			active_audit[AUDIT_setgid]=0;
			break;
		case SYS_setregid:
			if(active_audit[AUDIT_setregid]==0) return;
			sys_call_table[SYS_setregid]=orig_setregid;
			active_audit[AUDIT_setregid]=0;
			break;
		case SYS_setresgid:
			if(active_audit[AUDIT_setresgid]==0) return;
			sys_call_table[SYS_setresgid]=orig_setresgid;
			active_audit[AUDIT_setresgid]=0;
			break;
		case SYS_setgid32:
			if(active_audit[AUDIT_setgid32]==0) return;
			sys_call_table[SYS_setgid32]=orig_setgid32;
			active_audit[AUDIT_setgid32]=0;
			break;
		case SYS_setregid32:
			if(active_audit[AUDIT_setregid32]==0) return;
			sys_call_table[SYS_setregid32]=orig_setregid32;
			active_audit[AUDIT_setregid32]=0;
			break;
		case SYS_setresgid32:
			if(active_audit[AUDIT_setresgid32]==0) return;
			sys_call_table[SYS_setresgid32]=orig_setresgid32;
			active_audit[AUDIT_setresgid32]=0;
			break;
		case SYS_truncate64:
			if(active_audit[AUDIT_truncate64]==0) return;
			sys_call_table[SYS_truncate64]=orig_truncate64;
			active_audit[AUDIT_truncate64]=0;
			break;
		case SYS_socketcall:
			if(active_audit[AUDIT_socketcall]==0) return;
			sys_call_table[SYS_socketcall]=orig_socketcall;
			active_audit[AUDIT_socketcall]=0;
			break;
		case SYS_create_module:
			if(active_audit[AUDIT_create_module]==0) return;
			sys_call_table[SYS_create_module]=orig_create_module;
			active_audit[AUDIT_create_module]=0;
			break;
		//case SYS_delete_module:
		//	if(active_audit[AUDIT_delete_module]==0) return;
		//	sys_call_table[SYS_delete_module]=orig_delete_module;
		//	active_audit[AUDIT_delete_module]=0;
		//	break;
//		case SYS_ftruncate:
//			if(active_audit[AUDIT_ftruncate]==0) return;
//			sys_call_table[SYS_ftruncate]=orig_ftruncate;
//			active_audit[AUDIT_ftruncate]=0;
//			break;
//		case SYS_fchmod:
//			if(active_audit[AUDIT_fchmod]==0) return;
//			sys_call_table[SYS_fchmod]=orig_fchmod;
//			active_audit[AUDIT_fchmod]=0;
//			break;
//		case SYS_ftruncate64:
//			if(active_audit[AUDIT_ftruncate64]==0) return;
//			sys_call_table[SYS_ftruncate64]=orig_ftruncate64;
//			active_audit[AUDIT_ftruncate64]=0;
//			break;
//		case SYS_fchown:
//			if(active_audit[AUDIT_fchown]==0) return;
//			sys_call_table[SYS_fchown]=orig_fchown;
//			active_audit[AUDIT_fchown]=0;
//			break;
	}
}


////////////////////////////////////////////////////////////////////////////////
// Control the reporting of the audit event.
////////////////////////////////////////////////////////////////////////////////

int audit_event(void * event, int userset)
{
	int class=0;
	static struct timeval time_temp;	// to get the current time
	static null_class * any_event;
	int signal;
	signal=1;							// Signal the daemon

	any_event = event;

	// No use auditing anything unless the auditd is listening.
	if(auditdaemon_task_struct == NULL || audit_device_open == 0)
		return -ENODEV;

	// Reset both counts if we are close to maxint, otherwise we will have strange statistics.
	if(total_events>=(INT_MAX - 1)) {
		total_events=0;
		lost_events=0;
	}
	total_events++;

	// The routine that sends data to us should have filled in
	// the event_id in the header_token.
	// And the returncode should also be known.
	class = any_event->t_header.event_class;

	// fill in the time of the event
	// has to be when locked because otherwise we could interrupt and that may
	// generate an audited event - the times would then be out of order
	// Lock the command execution path until audit_spin_lock_irqrestore
	spin_lock_irqsave(&audit_spin_lock, flags);
	do_gettimeofday(&time_temp);
	// Allow the kernel to continue execution, and handle signals.
	spin_unlock_irqrestore(&audit_spin_lock, flags);

	any_event->t_header.time.tv_sec = time_temp.tv_sec;
	any_event->t_header.time.tv_usec = time_temp.tv_usec;

	// If userid is already set, don't grab it again.
	if(!userset) {
		// Get the current user data
		// This will be the same for any event.
		any_event->t_header.user_id=current->uid;
		any_event->t_header.group_id=current->gid;
		any_event->t_header.euser_id=current->euid;
		any_event->t_header.egroup_id=current->egid;
	}

	if(class == AUDIT_CLASS_IO)
	{
		// do somethin with the data.
		if(!write_io((io_class *)event))
		{
			// Couldnt write it out. Missed this event.
			signal=0;
			lost_events++;
		}
	} else if (class == AUDIT_CLASS_EXEC) {
		if(!write_ex((ex_class *)event))
		{
			// Couldnt write it out. Missed this event.
			signal=0;
			lost_events++;
		}
	} else if (class == AUDIT_CLASS_PC) {
		if(!write_pc((pc_class *)event))
		{
			// Couldnt write it out. Missed this event.
			signal=0;
			lost_events++;
		}
	} else if (class == AUDIT_CLASS_CH) {
		if(!write_ch((ch_class *)event))
		{
			// Couldnt write it out. Missed this event.
			signal=0;
			lost_events++;
		}
	} else if (class == AUDIT_CLASS_CP) {
		if(!write_cp((cp_class *)event))
		{
			// Couldnt write it out. Missed this event.
			signal=0;
			lost_events++;
		}
	} else if (class == AUDIT_CLASS_SU) {
		if(!write_su((su_class *)event))
		{
			// Couldnt write it out. Missed this event.
			signal=0;
			lost_events++;
		}
	} else if (class == AUDIT_CLASS_NET) {
		if(!write_nt((nt_class *)event))
		{
			// Couldnt write it out. Missed this event.
			signal=0;
			lost_events++;
		}
	} else if (class == AUDIT_CLASS_AD) {
		if(!write_ad((ad_class *)event))
		{
			// Couldnt write it out. Missed this event.
			signal=0;
			lost_events++;
		}
	} else {
		printk("Cannot handle this class yet.\n");
		signal=0;
		// Add 1 to the lost events tally
		lost_events++;
	}

	if(signal)
		signal_auditd();
	
	return(1); // no event handler
}

////////////////////////////////////////////////////////////////////////////////
// Safely write out the data to our buffer
//
// This routine should be audit_spin_locked for IRQ because it relies on variables
// that may be changed when the read routine is signaled/executed (which may
// happen half way through this routine without locking)
//	storage_buffer_start, write_position
//
// WARNING: DO NOT CHANGE OR RELY ON THE VARIABLES ABOVE UNLESS YOU ARE
//		  IN A SPIN_LOCKED STATE - THEY MAY BE CORRUPTED BY THE
//		  AUDIT_READ MODULE, WHICH IS CALLED BY SIGNAL/INTERRUPT
//
////////////////////////////////////////////////////////////////////////////////
int write_event(void * event, int token_size)
{
	int bytes_free=0;
	int contiguous_bytes_avail=0;
	int bytes_remaining=0;
	
	struct timeval starttime;	
	void *eventpointer;
	Node * nodepointer;
	
	
	eventpointer=kmalloc(token_size,GFP_KERNEL);

	if(eventpointer == (void *)NULL) {
		// Not enough RAM available aparently.
		spin_unlock_irqrestore(&audit_spin_lock, flags);
		return(0);
	}
		
	nodepointer=(Node *)kmalloc(sizeof(Node),GFP_KERNEL);
	if(nodepointer == (Node *)NULL) {
		// Not enough RAM available aparently.
		// Free the event pointer if we've got this far.
		kfree(eventpointer);
		spin_unlock_irqrestore(&audit_spin_lock, flags);
		return(0);
	}
	
	memcpy(eventpointer,event,token_size);

	// Don't allow interrupts to occur while we are here.
	spin_lock_irqsave(&audit_spin_lock, flags);

	if(list_tail != (Node *)NULL) {
		list_tail->next=nodepointer;
	}
	list_tail=nodepointer;
	
	// Is this the first node in our linked list?
	if(list_head == (Node *) NULL) {
		list_head=nodepointer;
	}
	
	nodepointer->location=eventpointer;
	nodepointer->size=token_size;
	nodepointer->next=(Node *) NULL;	// This is the end.
	
	CURRENTBUFFER+=(token_size+sizeof(Node));
	
	spin_unlock_irqrestore(&audit_spin_lock, flags);
	
	return(1);
}

////////////////////////////////////////////////////////////////////////////////
// The following section contains the broad write processes for each class
// of audit event.
////////////////////////////////////////////////////////////////////////////////

// Write input/output style events out to our buffer.
int write_io(io_class * event)
{
	char tempbuffer[MAX_PATH];
	// Set any other io related stuff

	// Get the current working directory.
	// New for kernel 2.4 - check if pwdmnt is appropriate. Alternatively: rootmnt/altrootmnt
	strncpy(event->t_pwd.path,d_path(current->fs->pwd,current->fs->rootmnt,tempbuffer,MAX_PATH),MAX_PATH);

	// Set the size to io_class minus header_token.
	event->t_header.event_size=sizeof(io_class) - sizeof(header_token);

	return(write_event(event,sizeof(io_class)));
}

// Write input/output chown style events out to our buffer.
int write_ch(ch_class * event)
{
	char tempbuffer[MAX_PATH];
	// Set any other ch related stuff

	// Get the current working directory.
	strncpy(event->t_pwd.path,d_path(current->fs->pwd,current->fs->rootmnt,tempbuffer,MAX_PATH),MAX_PATH);

	// Set the size to ch_class minus header_token.
	event->t_header.event_size=sizeof(ch_class) - sizeof(header_token);

	return(write_event(event,sizeof(ch_class)));
}

// Write execution style events out to our buffer.
int write_ex(ex_class * event)
{
	char tempbuffer[MAX_PATH];

	strncpy(event->t_pwd.path,d_path(current->fs->pwd,current->fs->rootmnt,tempbuffer,MAX_PATH),MAX_PATH);

	// Set the size to ex_class minus header_token.
	event->t_header.event_size=sizeof(ex_class) - sizeof(header_token);

	return(write_event(event,sizeof(ex_class)));
}

// Write process control style events out to our buffer.
int write_pc(pc_class * event)
{
	// Set the size to pc_class minus header_token.
	event->t_header.event_size=sizeof(pc_class) - sizeof(header_token);
	
	return(write_event(event,sizeof(pc_class)));
}

// Write input/output style events with a double path out to our buffer.
int write_cp(cp_class * event)
{
	char tempbuffer[MAX_PATH];

	// Get the current working directory.
	strncpy(event->t_pwd.path,d_path(current->fs->pwd,current->fs->rootmnt,tempbuffer,MAX_PATH),MAX_PATH);

	// Set the size to cp_class minus header_token.
	event->t_header.event_size=sizeof(cp_class) - sizeof(header_token);

	return(write_event(event,sizeof(cp_class)));
}

// Write su style events out to our buffer.
int write_su(su_class * event)
{
	// Set the size to su_class minus header_token.
	event->t_header.event_size=sizeof(su_class) - sizeof(header_token);
	
	return(write_event(event,sizeof(su_class)));
}

// Write net style events out to our buffer.
int write_nt(nt_class * event)
{
	// Set the size to su_class minus header_token.
	event->t_header.event_size=sizeof(nt_class) - sizeof(header_token);
	
	return(write_event(event,sizeof(nt_class)));
}

// Write admin style events out to our buffer.
int write_ad(ad_class * event)
{
	// Set the size to su_class minus header_token.
	event->t_header.event_size=sizeof(ad_class) - sizeof(header_token);
	
	return(write_event(event,sizeof(ad_class)));
}

////////////////////////////////////////////////////////////////////////////////
// The following section contains the modified system calls that include
// the auditing that we require.
// Note that the kernel will be IRQ spin-locked once the audit_event routine
// is called.
////////////////////////////////////////////////////////////////////////////////

// Audit file opens.
asmlinkage int audit_open(const char *pathname, int flag, mode_t mode)
{
	io_class	io_event;

	io_event.t_return.returncode=orig_open(pathname, flag, mode);

	// If no daemon, or the daemon has generated the event.
	if((!AUDIT_IS_RUNNING) || (AUDIT_IS_RUNNING && (current->pid == auditdaemon_task_struct->pid)))
	{
		return(io_event.t_return.returncode);
	}

	io_event.t_header.event_class=AUDIT_CLASS_IO;
	io_event.t_header.event_id=SYS_open;

	// Process data
	io_event.t_process.pid=current->pid;
	io_event.t_process.name[0]='\0';
	strncpy(io_event.t_process.name,current->comm,MAXCOMMAND);

	io_event.t_path.path[0]='\0';
	if(pathname) {
		strncpy_from_user(io_event.t_path.path,pathname,MAX_PATH);
	}
	io_event.t_attributes.mode=flag;
	io_event.t_attributes.createmode=mode;

	// Everything else can be handled by the generic event handler.
	audit_event(&io_event,0);

	return(io_event.t_return.returncode);
}

// Audit file opens (create only).
asmlinkage int audit_creat(const char *pathname, mode_t mode)
{
	io_class	io_event;

	io_event.t_return.returncode=orig_creat(pathname, mode);

	// If no daemon, or the daemon has generated the event.
	if((!AUDIT_IS_RUNNING) || (AUDIT_IS_RUNNING && (current->pid == auditdaemon_task_struct->pid)))
	{
		return(io_event.t_return.returncode);
	}

	io_event.t_header.event_class=AUDIT_CLASS_IO;
	io_event.t_header.event_id=SYS_creat;

	// Process data
	io_event.t_process.pid=current->pid;
	io_event.t_process.name[0]='\0';
	strncpy(io_event.t_process.name,current->comm,MAXCOMMAND);

	io_event.t_path.path[0]='\0';
	if(pathname) {
		strncpy_from_user(io_event.t_path.path,pathname,MAX_PATH);
	}
	io_event.t_attributes.mode=O_CREAT|O_WRONLY|O_TRUNC;
	io_event.t_attributes.createmode=mode;

	// Everything else can be handled by the generic event handler.
	audit_event(&io_event,0);

	return(io_event.t_return.returncode);
}

// Audit execve command executions
// Execve is not handled quite the same as other audit events.
int audit_execve(struct pt_regs regs)
{
	ex_class	ex_event;

	char *filename=NULL;
	char **argv=NULL;

	unsigned int delimiter_required=0;

	// Not really sure I need to getname here.
	// After-all, I'm not sending this to user-space yet.
	// I'll keep it in for safety.
	filename = getname((char*)regs.ebx);
	argv = (char **)regs.ecx;

	// If audit is not active, don't go any further.
	if((!AUDIT_IS_RUNNING) || (AUDIT_IS_RUNNING && (current->pid == auditdaemon_task_struct->pid)))
	{
		ex_event.t_return.returncode=do_execve(filename, (char**)regs.ecx, (char**)regs.edx, &regs);
		putname(filename);
		return(ex_event.t_return.returncode);
	}

	// Construct the argument list
	// and shoehorn it into exec_args

	ex_event.t_execargs.args[0]='\0';
	while(*argv != NULL)
	{
		// Allow for the delimiter
		if((strlen(ex_event.t_execargs.args) + strlen(*argv)) < MAX_PATH)
		{
			if(!delimiter_required)
			{
				delimiter_required=1;
			} else {
				strcat(ex_event.t_execargs.args," ");
			}
			strcat(ex_event.t_execargs.args,*argv);
		} else {
			break;
		}
		argv++;
	}

	// Process data
	ex_event.t_process.pid=current->pid;
	ex_event.t_process.name[0]='\0';
	strncpy(ex_event.t_process.name,current->comm,MAXCOMMAND);
	
	ex_event.t_return.returncode = do_execve(filename, (char**)regs.ecx, (char**)regs.edx, &regs);
	ex_event.t_path.path[0]='\0';
	strncpy(ex_event.t_path.path,filename,MAX_PATH);

	putname(filename);

	ex_event.t_header.event_class=AUDIT_CLASS_EXEC;
	ex_event.t_header.event_id=SYS_execve;

	// Everything else can be handled by the generic event handler.
	audit_event(&ex_event,0);

	return(ex_event.t_return.returncode);
}


// Audit process exits.
// This may be useful if we want to audit login/logouts - because
// as far as I can work out, the only way we can do it is to watch for
// execve/exit of /bin/login (without hacking the kernel).
//
// Note: current->comm is a 16 character representation of the executed command.
asmlinkage int audit_exit(int status)
{
	pc_class	pc_event;

	// If no daemon, or the daemon has generated the event.
	if((!AUDIT_IS_RUNNING) || (AUDIT_IS_RUNNING && (current->pid == auditdaemon_task_struct->pid)))
	{
		return orig_exit(status);
	}

	pc_event.t_return.returncode=status;

	pc_event.t_header.event_class=AUDIT_CLASS_PC;
	pc_event.t_header.event_id=SYS_exit;

	pc_event.t_process.pid=current->pid;
	// Zero the string just in case this fails.
	pc_event.t_process.name[0]='\0';
	strncpy(pc_event.t_process.name,current->comm,MAXCOMMAND);

	// Everything else can be handled by the generic event handler.
	audit_event(&pc_event,0);

	// Now continue on with the normal exit.
	return orig_exit(status);
}

// Audit mkdir directory creations.
asmlinkage int audit_mkdir(const char *path, mode_t mode)
{
	io_class	io_event;
	int		returncode;

	// If no daemon, or the daemon has generated the event.
	if((!AUDIT_IS_RUNNING) || (AUDIT_IS_RUNNING && (current->pid == auditdaemon_task_struct->pid)))
	{
		return orig_mkdir(path,mode);
	}

	returncode=orig_mkdir(path,mode);
	
	io_event.t_path.path[0]='\0';
	if(path) {
		strncpy_from_user(io_event.t_path.path,path,MAX_PATH);
	}
	io_event.t_path.path[MAX_PATH-1]='\0';

	io_event.t_return.returncode=returncode;

	io_event.t_header.event_class=AUDIT_CLASS_IO;
	io_event.t_header.event_id=SYS_mkdir;
	io_event.t_process.pid=current->pid;
	io_event.t_process.name[0]='\0';
	strncpy(io_event.t_process.name,current->comm,MAXCOMMAND);

	io_event.t_attributes.mode=0;
	io_event.t_attributes.createmode=(unsigned long)mode;

	// Everything else can be handled by the generic event handler.
	audit_event(&io_event,0);

	// Now continue on with the normal exit.
	return returncode;
}

// audit file unlinks
asmlinkage int	audit_unlink (const char *path)
{
	io_class	io_event;

	// If no daemon, or the daemon has generated the event.
	if((!AUDIT_IS_RUNNING) || (AUDIT_IS_RUNNING && (current->pid == auditdaemon_task_struct->pid)))
	{
		return orig_unlink(path);
	}

	io_event.t_return.returncode=orig_unlink(path);

	io_event.t_header.event_class=AUDIT_CLASS_IO;
	io_event.t_header.event_id=SYS_unlink;
	io_event.t_process.pid=current->pid;
	io_event.t_process.name[0]='\0';
	strncpy(io_event.t_process.name,current->comm,MAXCOMMAND);

	// Store the path
	// zero the path, just in case strncpy fails.
	io_event.t_path.path[0]='\0';
	if(path) {
		strncpy_from_user(io_event.t_path.path,path,MAX_PATH);
	}

	// Everything else can be handled by the generic event handler.
	audit_event(&io_event,0);

	// Now continue on with the normal exit.
	return io_event.t_return.returncode;
}

// Create a special file/directory
asmlinkage int audit_mknod (const char *pathname, mode_t mode, dev_t dev)
{
	io_class	io_event;

	io_event.t_return.returncode=orig_mknod(pathname, mode, dev);

	// If no daemon, or the daemon has generated the event.
	if((!AUDIT_IS_RUNNING) || (AUDIT_IS_RUNNING && (current->pid == auditdaemon_task_struct->pid)))
	{
		return(io_event.t_return.returncode);
	}

	io_event.t_header.event_class=AUDIT_CLASS_IO;
	io_event.t_header.event_id=SYS_mknod;

	// Process data
	io_event.t_process.pid=current->pid;
	io_event.t_process.name[0]='\0';
	strncpy(io_event.t_process.name,current->comm,MAXCOMMAND);

	io_event.t_path.path[0]='\0';
	if(pathname) {
		strncpy_from_user(io_event.t_path.path,pathname,MAX_PATH);
	}
	io_event.t_attributes.mode=(int)mode;
	io_event.t_attributes.createmode=(unsigned long)dev;

	// Everything else can be handled by the generic event handler.
	audit_event(&io_event,0);

	return(io_event.t_return.returncode);
}

// Audit rmdir directory removals.
asmlinkage int audit_rmdir(const char *path)
{
	io_class	io_event;

	// If no daemon, or the daemon has generated the event.
	if((!AUDIT_IS_RUNNING) || (AUDIT_IS_RUNNING && (current->pid == auditdaemon_task_struct->pid)))
	{
		return orig_rmdir(path);
	}

	io_event.t_return.returncode=orig_rmdir(path);

	io_event.t_header.event_class=AUDIT_CLASS_IO;
	io_event.t_header.event_id=SYS_rmdir;
	io_event.t_process.pid=current->pid;
	io_event.t_process.name[0]='\0';
	strncpy(io_event.t_process.name,current->comm,MAXCOMMAND);

	// Store the path
	io_event.t_path.path[0]='\0';
	if(path) {
		strncpy_from_user(io_event.t_path.path,path,MAX_PATH);
	}

	// Everything else can be handled by the generic event handler.
	audit_event(&io_event,0);
	
	return io_event.t_return.returncode;
}

// Audit change ownership
asmlinkage int audit_chown(const char *path, uid_t owner, gid_t group)
{
	ch_class	ch_event;

	// If no daemon, or the daemon has generated the event.
	if((!AUDIT_IS_RUNNING) || (AUDIT_IS_RUNNING && (current->pid == auditdaemon_task_struct->pid)))
	{
		return orig_chown(path,owner,group);
	}

	ch_event.t_return.returncode=orig_chown(path,owner,group);

	ch_event.t_header.event_class=AUDIT_CLASS_CH;
	ch_event.t_header.event_id=SYS_chown;
	ch_event.t_process.pid=current->pid;
	ch_event.t_process.name[0]='\0';
	strncpy(ch_event.t_process.name,current->comm,MAXCOMMAND);

	ch_event.t_owner.owner=(int) owner;
	ch_event.t_owner.group=(int) group;

	// Store the path
	ch_event.t_path.path[0]='\0';
	if(path) {
		strncpy_from_user(ch_event.t_path.path,path,MAX_PATH);
	}

	// Everything else can be handled by the generic event handler.
	audit_event(&ch_event,0);

	// Now continue on with the normal exit.
	return ch_event.t_return.returncode;

}

// Same as chown, but does not follow symlinks.
asmlinkage int audit_lchown(const char *path, uid_t owner, gid_t group)
{
	ch_class	ch_event;

	// If no daemon, or the daemon has generated the event.
	if((!AUDIT_IS_RUNNING) || (AUDIT_IS_RUNNING && (current->pid == auditdaemon_task_struct->pid)))
	{
		return orig_lchown(path,owner,group);
	}

	ch_event.t_return.returncode=orig_lchown(path,owner,group);

	ch_event.t_header.event_class=AUDIT_CLASS_CH;
	ch_event.t_header.event_id=SYS_lchown;
	ch_event.t_process.pid=current->pid;
	ch_event.t_process.name[0]='\0';
	strncpy(ch_event.t_process.name,current->comm,MAXCOMMAND);

	ch_event.t_owner.owner=(int) owner;
	ch_event.t_owner.group=(int) group;

	// Store the path
	ch_event.t_path.path[0]='\0';
	if(path) {
		strncpy_from_user(ch_event.t_path.path,path,MAX_PATH);
	}

	// Everything else can be handled by the generic event handler.
	audit_event(&ch_event,0);

	// Now continue on with the normal exit.
	return ch_event.t_return.returncode;
}

// Change ownership
asmlinkage int audit_chown32(const char *path, uid_t owner, gid_t group)
{
	ch_class	ch_event;

	// If no daemon, or the daemon has generated the event.
	if((!AUDIT_IS_RUNNING) || (AUDIT_IS_RUNNING && (current->pid == auditdaemon_task_struct->pid)))
	{
		return orig_chown32(path,owner,group);
	}

	ch_event.t_return.returncode=orig_chown32(path,owner,group);

	ch_event.t_header.event_class=AUDIT_CLASS_CH;
	ch_event.t_header.event_id=SYS_chown32;

	ch_event.t_process.pid=current->pid;
	ch_event.t_process.name[0]='\0';
	strncpy(ch_event.t_process.name,current->comm,MAXCOMMAND);

	ch_event.t_owner.owner=(int) owner;
	ch_event.t_owner.group=(int) group;

	// Store the path
	ch_event.t_path.path[0]='\0';
	if(path) {
		strncpy_from_user(ch_event.t_path.path,path,MAX_PATH);
	}

	// Everything else can be handled by the generic event handler.
	audit_event(&ch_event,0);

	// Now continue on with the normal exit.
	return ch_event.t_return.returncode;

}

// Same as chown, but does not follow symlinks.
asmlinkage int audit_lchown32(const char *path, uid_t owner, gid_t group)
{
	ch_class	ch_event;

	// If no daemon, or the daemon has generated the event.
	if((!AUDIT_IS_RUNNING) || (AUDIT_IS_RUNNING && (current->pid == auditdaemon_task_struct->pid)))
	{
		return orig_lchown32(path,owner,group);
	}

	ch_event.t_return.returncode=orig_lchown32(path,owner,group);

	ch_event.t_header.event_class=AUDIT_CLASS_CH;
	ch_event.t_header.event_id=SYS_lchown32;
	ch_event.t_process.pid=current->pid;
	ch_event.t_process.name[0]='\0';
	strncpy(ch_event.t_process.name,current->comm,MAXCOMMAND);

	ch_event.t_owner.owner=(int) owner;
	ch_event.t_owner.group=(int) group;

	// Store the path
	ch_event.t_path.path[0]='\0';
	if(path) {
		strncpy_from_user(ch_event.t_path.path,path,MAX_PATH);
	}

	// Everything else can be handled by the generic event handler.
	audit_event(&ch_event,0);

	// Now continue on with the normal exit.
	return ch_event.t_return.returncode;

}


// Change the mode of a file or directory
asmlinkage int audit_chmod (const char *pathname, mode_t mode)
{
	io_class	io_event;

	io_event.t_return.returncode=orig_chmod(pathname, mode);

	// If no daemon, or the daemon has generated the event.
	if((!AUDIT_IS_RUNNING) || (AUDIT_IS_RUNNING && (current->pid == auditdaemon_task_struct->pid)))
	{
		return(io_event.t_return.returncode);
	}

	io_event.t_header.event_class=AUDIT_CLASS_IO;
	io_event.t_header.event_id=SYS_chmod;

	// Process data
	io_event.t_process.pid=current->pid;
	io_event.t_process.name[0]='\0';
	strncpy(io_event.t_process.name,current->comm,MAXCOMMAND);

	io_event.t_path.path[0]='\0';
	if(pathname) {
		strncpy_from_user(io_event.t_path.path,pathname,MAX_PATH);
	}
	io_event.t_attributes.mode=(int)NULL;
	io_event.t_attributes.createmode=(unsigned long)mode;

	// Everything else can be handled by the generic event handler.
	audit_event(&io_event,0);

	return(io_event.t_return.returncode);
}

// Create a symbolic link
asmlinkage int audit_symlink (const char *oldpath, const char *newpath)
{
	cp_class	cp_event;

	cp_event.t_return.returncode=orig_symlink(oldpath, newpath);

	// If no daemon, or the daemon has generated the event.
	if((!AUDIT_IS_RUNNING) || (AUDIT_IS_RUNNING && (current->pid == auditdaemon_task_struct->pid)))
	{
		return(cp_event.t_return.returncode);
	}

	cp_event.t_header.event_class=AUDIT_CLASS_CP;
	cp_event.t_header.event_id=SYS_symlink;

	// Process data
	cp_event.t_process.pid=current->pid;
	cp_event.t_process.name[0]='\0';
	strncpy(cp_event.t_process.name,current->comm,MAXCOMMAND);

	cp_event.t_sourcepath.path[0]='\0';
	if(oldpath) {
		strncpy_from_user(cp_event.t_sourcepath.path,oldpath,MAX_PATH);
	}
	cp_event.t_destpath.path[0]='\0';
	if(newpath) {
		strncpy_from_user(cp_event.t_destpath.path,newpath,MAX_PATH);
	}

	// Everything else can be handled by the generic event handler.
	audit_event(&cp_event,0);

	return(cp_event.t_return.returncode);
}

// Create a hard link
asmlinkage int audit_link (const char *oldpath, const char *newpath)
{
	cp_class	cp_event;

	cp_event.t_return.returncode=orig_link(oldpath, newpath);

	// If no daemon, or the daemon has generated the event.
	if((!AUDIT_IS_RUNNING) || (AUDIT_IS_RUNNING && (current->pid == auditdaemon_task_struct->pid)))
	{
		return(cp_event.t_return.returncode);
	}

	cp_event.t_header.event_class=AUDIT_CLASS_CP;
	cp_event.t_header.event_id=SYS_link;

	// Process data
	cp_event.t_process.pid=current->pid;
	cp_event.t_process.name[0]='\0';
	strncpy(cp_event.t_process.name,current->comm,MAXCOMMAND);

	cp_event.t_sourcepath.path[0]='\0';
	if(oldpath) {
		strncpy_from_user(cp_event.t_sourcepath.path,oldpath,MAX_PATH);
	}
	cp_event.t_destpath.path[0]='\0';
	if(newpath) {
		strncpy_from_user(cp_event.t_destpath.path,newpath,MAX_PATH);
	}

	// Everything else can be handled by the generic event handler.
	audit_event(&cp_event,0);

	return(cp_event.t_return.returncode);
}

// rename a file/directory
asmlinkage int audit_rename (const char *oldpath, const char *newpath)
{
	cp_class	cp_event;

	cp_event.t_return.returncode=orig_rename(oldpath, newpath);

	// If no daemon, or the daemon has generated the event.
	if((!AUDIT_IS_RUNNING) || (AUDIT_IS_RUNNING && (current->pid == auditdaemon_task_struct->pid)))
	{
		return(cp_event.t_return.returncode);
	}

	cp_event.t_header.event_class=AUDIT_CLASS_CP;
	cp_event.t_header.event_id=SYS_rename;

	// Process data
	cp_event.t_process.pid=current->pid;
	cp_event.t_process.name[0]='\0';
	strncpy(cp_event.t_process.name,current->comm,MAXCOMMAND);

	cp_event.t_sourcepath.path[0]='\0';
	if(oldpath) {
		strncpy_from_user(cp_event.t_sourcepath.path,oldpath,MAX_PATH);
	}
	cp_event.t_destpath.path[0]='\0';
	if(newpath) {
		strncpy_from_user(cp_event.t_destpath.path,newpath,MAX_PATH);
	}

	// Everything else can be handled by the generic event handler.
	audit_event(&cp_event,0);

	return(cp_event.t_return.returncode);
}

// Would like to audit reboots here.
// NOTE: this only audits when the reboot() system call is issued. There are many other
//       ways to reboot a system (eg: kill -9 1) - this will NOT catch these.
//       In fact, a normal reboot from the gdm login manager (for example), does not
//       seem to call the reboot() call. So this call may not be particularly useful.
// Only question is: does reboot return only after the system goes down?
// (in which case, I'll have to do the audit, fake the return code, and
// return with the orig_reboot().
// After all... we don't want to try and write stuff after filesystems have been unmounted.
asmlinkage int audit_reboot(int magic, int magic2, int flag, void *arg)
{
	pc_class	pc_event;

	pc_event.t_return.returncode=orig_reboot(magic, magic2, flag, arg);

	// If no daemon, or the daemon has generated the event.
	// Or if the reboot flags are just disabling or reenabling the CAD key
	if((!AUDIT_IS_RUNNING) || (AUDIT_IS_RUNNING && (current->pid == auditdaemon_task_struct->pid)) ||
	flag == LINUX_REBOOT_CMD_CAD_ON || flag == LINUX_REBOOT_CMD_CAD_OFF)
	{
		return(pc_event.t_return.returncode);
	}

	pc_event.t_header.event_class=AUDIT_CLASS_PC;
	pc_event.t_header.event_id=SYS_reboot;

	// Process data
	pc_event.t_process.pid=current->pid;
	pc_event.t_process.name[0]='\0';
	strncpy(pc_event.t_process.name,current->comm,MAXCOMMAND);

	// Everything else can be handled by the generic event handler.
	audit_event(&pc_event,0);

	return(pc_event.t_return.returncode);
}

// audit file truncates.
// Note that off_t can be shoehorned into a unsigned long for auditing.
asmlinkage int	audit_truncate (const char *path, off_t length)
{
	io_class	io_event;

	// If no daemon, or the daemon has generated the event.
	if((!AUDIT_IS_RUNNING) || (AUDIT_IS_RUNNING && (current->pid == auditdaemon_task_struct->pid)))
	{
		return orig_truncate(path,length);
	}

	io_event.t_return.returncode=orig_truncate(path,length);

	io_event.t_header.event_class=AUDIT_CLASS_IO;
	io_event.t_header.event_id=SYS_truncate;
	io_event.t_process.pid=current->pid;
	io_event.t_process.name[0]='\0';
	strncpy(io_event.t_process.name,current->comm,MAXCOMMAND);

	// Store the path
	// zero the path, just in case strncpy fails.
	io_event.t_path.path[0]='\0';
	if(path) {
		strncpy_from_user(io_event.t_path.path,path,MAX_PATH);
	}
	
	// Save off the offset.
	io_event.t_attributes.createmode=(unsigned long)length;

	// Everything else can be handled by the generic event handler.
	audit_event(&io_event,0);

	// Now continue on with the normal exit.
	return io_event.t_return.returncode;
}

// audit root changes.
asmlinkage int	audit_chroot (const char *path)
{
	io_class	io_event;

	// If no daemon, or the daemon has generated the event.
	if((!AUDIT_IS_RUNNING) || (AUDIT_IS_RUNNING && (current->pid == auditdaemon_task_struct->pid)))
	{
		return orig_chroot(path);
	}

	io_event.t_return.returncode=orig_chroot(path);

	io_event.t_header.event_class=AUDIT_CLASS_IO;
	io_event.t_header.event_id=SYS_chroot;
	io_event.t_process.pid=current->pid;
	io_event.t_process.name[0]='\0';
	strncpy(io_event.t_process.name,current->comm,MAXCOMMAND);

	// Store the path
	// zero the path, just in case strncpy fails.
	io_event.t_path.path[0]='\0';
	if(path) {
		strncpy_from_user(io_event.t_path.path,path,MAX_PATH);
	}

	// Everything else can be handled by the generic event handler.
	audit_event(&io_event,0);

	// Now continue on with the normal exit.
	return io_event.t_return.returncode;
}

// Setuid system calls
asmlinkage int	audit_setuid (uid_t uid)
{
	su_class	su_event;

	// If no daemon, or the daemon has generated the event.
	if((!AUDIT_IS_RUNNING) || (AUDIT_IS_RUNNING && (current->pid == auditdaemon_task_struct->pid)))
	{
		return orig_setuid(uid);
	}

	// Set the current id data from the pre-system call user data
	su_event.t_header.user_id=current->uid;
	su_event.t_header.group_id=current->gid;
	su_event.t_header.euser_id=current->euid;
	su_event.t_header.egroup_id=current->egid;

	su_event.t_return.returncode=orig_setuid(uid);

	su_event.t_header.event_class=AUDIT_CLASS_SU;
	su_event.t_header.event_id=SYS_setuid;
	su_event.t_process.pid=current->pid;
	su_event.t_process.name[0]='\0';
	strncpy(su_event.t_process.name,current->comm,MAXCOMMAND);

	su_event.t_target.id=uid;

	// Everything else can be handled by the generic event handler.
	audit_event(&su_event,1);

	// Now continue on with the normal exit.
	return su_event.t_return.returncode;
}

asmlinkage int	audit_setreuid (uid_t ruid, uid_t euid)
{
	su_class	su_event;

	// If no daemon, or the daemon has generated the event.
	if((!AUDIT_IS_RUNNING) || (AUDIT_IS_RUNNING && (current->pid == auditdaemon_task_struct->pid)))
	{
		return orig_setreuid(ruid,euid);
	}

	// Set the current id data from the pre-system call user data
	su_event.t_header.user_id=current->uid;
	su_event.t_header.group_id=current->gid;
	su_event.t_header.euser_id=current->euid;
	su_event.t_header.egroup_id=current->egid;

	su_event.t_return.returncode=orig_setreuid(ruid,euid);

	su_event.t_header.event_class=AUDIT_CLASS_SU;
	su_event.t_header.event_id=SYS_setreuid;
	su_event.t_process.pid=current->pid;
	su_event.t_process.name[0]='\0';
	strncpy(su_event.t_process.name,current->comm,MAXCOMMAND);

	su_event.t_target.id=euid;
	su_event.t_target.rid=ruid;

	// Everything else can be handled by the generic event handler.
	audit_event(&su_event,1);

	// Now continue on with the normal exit.
	return su_event.t_return.returncode;
}

// Set real, effective and saved user ID
asmlinkage int	audit_setresuid (uid_t ruid, uid_t euid, uid_t suid)
{
	su_class	su_event;

	// If no daemon, or the daemon has generated the event.
	if((!AUDIT_IS_RUNNING) || (AUDIT_IS_RUNNING && (current->pid == auditdaemon_task_struct->pid)))
	{
		return orig_setresuid(ruid,euid,suid);
	}

	// Set the current id data from the pre-system call user data
	su_event.t_header.user_id=current->uid;
	su_event.t_header.group_id=current->gid;
	su_event.t_header.euser_id=current->euid;
	su_event.t_header.egroup_id=current->egid;

	su_event.t_return.returncode=orig_setresuid(ruid,euid,suid);

	su_event.t_header.event_class=AUDIT_CLASS_SU;
	su_event.t_header.event_id=SYS_setresuid;
	su_event.t_process.pid=current->pid;
	su_event.t_process.name[0]='\0';
	strncpy(su_event.t_process.name,current->comm,MAXCOMMAND);

	su_event.t_target.id=euid;
	su_event.t_target.rid=ruid;
	su_event.t_target.sid=suid;

	// Everything else can be handled by the generic event handler.
	audit_event(&su_event,1);

	// Now continue on with the normal exit.
	return su_event.t_return.returncode;
}

asmlinkage int	audit_setuid32 (uid_t uid)
{
	su_class	su_event;

	// If no daemon, or the daemon has generated the event.
	if((!AUDIT_IS_RUNNING) || (AUDIT_IS_RUNNING && (current->pid == auditdaemon_task_struct->pid)))
	{
		return orig_setuid32(uid);
	}

	// Set the current id data from the pre-system call user data
	su_event.t_header.user_id=current->uid;
	su_event.t_header.group_id=current->gid;
	su_event.t_header.euser_id=current->euid;
	su_event.t_header.egroup_id=current->egid;

	su_event.t_return.returncode=orig_setuid32(uid);

	su_event.t_header.event_class=AUDIT_CLASS_SU;
	su_event.t_header.event_id=SYS_setuid32;
	su_event.t_process.pid=current->pid;
	su_event.t_process.name[0]='\0';
	strncpy(su_event.t_process.name,current->comm,MAXCOMMAND);

	su_event.t_target.id=uid;
	su_event.t_target.rid=0;
	su_event.t_target.sid=0;

	// Everything else can be handled by the generic event handler.
	audit_event(&su_event,1);

	// Now continue on with the normal exit.
	return su_event.t_return.returncode;
}

asmlinkage int	audit_setreuid32 (uid_t ruid, uid_t euid)
{
	su_class	su_event;

	// If no daemon, or the daemon has generated the event.
	if((!AUDIT_IS_RUNNING) || (AUDIT_IS_RUNNING && (current->pid == auditdaemon_task_struct->pid)))
	{
		return orig_setreuid32(ruid,euid);
	}

	// Set the current id data from the pre-system call user data
	su_event.t_header.user_id=current->uid;
	su_event.t_header.group_id=current->gid;
	su_event.t_header.euser_id=current->euid;
	su_event.t_header.egroup_id=current->egid;

	su_event.t_return.returncode=orig_setreuid32(ruid,euid);

	su_event.t_header.event_class=AUDIT_CLASS_SU;
	su_event.t_header.event_id=SYS_setreuid32;
	su_event.t_process.pid=current->pid;
	su_event.t_process.name[0]='\0';
	strncpy(su_event.t_process.name,current->comm,MAXCOMMAND);

	su_event.t_target.id=euid;
	su_event.t_target.rid=ruid;
	su_event.t_target.sid=0;

	// Everything else can be handled by the generic event handler.
	audit_event(&su_event,1);

	// Now continue on with the normal exit.
	return su_event.t_return.returncode;
}

asmlinkage int	audit_setresuid32 (uid_t ruid, uid_t euid, uid_t suid)
{
	su_class	su_event;

	// If no daemon, or the daemon has generated the event.
	if((!AUDIT_IS_RUNNING) || (AUDIT_IS_RUNNING && (current->pid == auditdaemon_task_struct->pid)))
	{
		return orig_setresuid32(ruid,euid,suid);
	}

	// Set the current id data from the pre-system call user data
	su_event.t_header.user_id=current->uid;
	su_event.t_header.group_id=current->gid;
	su_event.t_header.euser_id=current->euid;
	su_event.t_header.egroup_id=current->egid;

	su_event.t_return.returncode=orig_setresuid32(ruid,euid,suid);

	su_event.t_header.event_class=AUDIT_CLASS_SU;
	su_event.t_header.event_id=SYS_setresuid32;
	su_event.t_process.pid=current->pid;
	su_event.t_process.name[0]='\0';
	strncpy(su_event.t_process.name,current->comm,MAXCOMMAND);

	su_event.t_target.id=euid;
	su_event.t_target.rid=ruid;
	su_event.t_target.sid=suid;

	// Everything else can be handled by the generic event handler.
	audit_event(&su_event,1);

	// Now continue on with the normal exit.
	return su_event.t_return.returncode;
}

asmlinkage int	audit_setgid (gid_t gid)
{
	su_class	su_event;

	// If no daemon, or the daemon has generated the event.
	if((!AUDIT_IS_RUNNING) || (AUDIT_IS_RUNNING && (current->pid == auditdaemon_task_struct->pid)))
	{
		return orig_setgid(gid);
	}

	// Set the current id data from the pre-system call user data
	su_event.t_header.user_id=current->uid;
	su_event.t_header.group_id=current->gid;
	su_event.t_header.euser_id=current->euid;
	su_event.t_header.egroup_id=current->egid;

	su_event.t_return.returncode=orig_setgid(gid);

	su_event.t_header.event_class=AUDIT_CLASS_SU;
	su_event.t_header.event_id=SYS_setgid;
	su_event.t_process.pid=current->pid;
	su_event.t_process.name[0]='\0';
	strncpy(su_event.t_process.name,current->comm,MAXCOMMAND);

	su_event.t_target.id=gid;

	// Everything else can be handled by the generic event handler.
	audit_event(&su_event,1);

	// Now continue on with the normal exit.
	return su_event.t_return.returncode;
}

asmlinkage int	audit_setregid (gid_t rgid, gid_t egid)
{
	su_class	su_event;

	// If no daemon, or the daemon has generated the event.
	if((!AUDIT_IS_RUNNING) || (AUDIT_IS_RUNNING && (current->pid == auditdaemon_task_struct->pid)))
	{
		return orig_setregid(rgid,egid);
	}

	// Set the current id data from the pre-system call user data
	su_event.t_header.user_id=current->uid;
	su_event.t_header.group_id=current->gid;
	su_event.t_header.euser_id=current->euid;
	su_event.t_header.egroup_id=current->egid;

	su_event.t_return.returncode=orig_setregid(rgid,egid);

	su_event.t_header.event_class=AUDIT_CLASS_SU;
	su_event.t_header.event_id=SYS_setregid;
	su_event.t_process.pid=current->pid;
	su_event.t_process.name[0]='\0';
	strncpy(su_event.t_process.name,current->comm,MAXCOMMAND);

	su_event.t_target.id=egid;
	su_event.t_target.rid=rgid;

	// Everything else can be handled by the generic event handler.
	audit_event(&su_event,1);

	// Now continue on with the normal exit.
	return su_event.t_return.returncode;
}

asmlinkage int	audit_setresgid (gid_t rgid, gid_t egid, gid_t sgid)
{
	su_class	su_event;

	// If no daemon, or the daemon has generated the event.
	if((!AUDIT_IS_RUNNING) || (AUDIT_IS_RUNNING && (current->pid == auditdaemon_task_struct->pid)))
	{
		return orig_setresgid(rgid,egid,sgid);
	}

	// Set the current id data from the pre-system call user data
	su_event.t_header.user_id=current->uid;
	su_event.t_header.group_id=current->gid;
	su_event.t_header.euser_id=current->euid;
	su_event.t_header.egroup_id=current->egid;

	su_event.t_return.returncode=orig_setresgid(rgid,egid,sgid);

	su_event.t_header.event_class=AUDIT_CLASS_SU;
	su_event.t_header.event_id=SYS_setresgid;
	su_event.t_process.pid=current->pid;
	su_event.t_process.name[0]='\0';
	strncpy(su_event.t_process.name,current->comm,MAXCOMMAND);

	su_event.t_target.id=egid;
	su_event.t_target.rid=rgid;
	su_event.t_target.sid=sgid;

	// Everything else can be handled by the generic event handler.
	audit_event(&su_event,1);

	// Now continue on with the normal exit.
	return su_event.t_return.returncode;
}

asmlinkage int	audit_setgid32 (gid_t gid)
{
	su_class	su_event;

	// If no daemon, or the daemon has generated the event.
	if((!AUDIT_IS_RUNNING) || (AUDIT_IS_RUNNING && (current->pid == auditdaemon_task_struct->pid)))
	{
		return orig_setgid32(gid);
	}

	// Set the current id data from the pre-system call user data
	su_event.t_header.user_id=current->uid;
	su_event.t_header.group_id=current->gid;
	su_event.t_header.euser_id=current->euid;
	su_event.t_header.egroup_id=current->egid;

	su_event.t_return.returncode=orig_setgid32(gid);

	su_event.t_header.event_class=AUDIT_CLASS_SU;
	su_event.t_header.event_id=SYS_setgid32;
	su_event.t_process.pid=current->pid;
	su_event.t_process.name[0]='\0';
	strncpy(su_event.t_process.name,current->comm,MAXCOMMAND);

	su_event.t_target.id=gid;
	su_event.t_target.rid=0;
	su_event.t_target.sid=0;

	// Everything else can be handled by the generic event handler.
	audit_event(&su_event,1);

	// Now continue on with the normal exit.
	return su_event.t_return.returncode;
}

asmlinkage int	audit_setregid32 (gid_t rgid, gid_t egid)
{
	su_class	su_event;

	// If no daemon, or the daemon has generated the event.
	if((!AUDIT_IS_RUNNING) || (AUDIT_IS_RUNNING && (current->pid == auditdaemon_task_struct->pid)))
	{
		return orig_setregid32(rgid,egid);
	}

	// Set the current id data from the pre-system call user data
	su_event.t_header.user_id=current->uid;
	su_event.t_header.group_id=current->gid;
	su_event.t_header.euser_id=current->euid;
	su_event.t_header.egroup_id=current->egid;

	su_event.t_return.returncode=orig_setregid32(rgid,egid);

	su_event.t_header.event_class=AUDIT_CLASS_SU;
	su_event.t_header.event_id=SYS_setregid32;
	su_event.t_process.pid=current->pid;
	su_event.t_process.name[0]='\0';
	strncpy(su_event.t_process.name,current->comm,MAXCOMMAND);

	su_event.t_target.id=egid;
	su_event.t_target.rid=rgid;
	su_event.t_target.sid=0;

	// Everything else can be handled by the generic event handler.
	audit_event(&su_event,1);

	// Now continue on with the normal exit.
	return su_event.t_return.returncode;
}

asmlinkage int	audit_setresgid32 (gid_t rgid, gid_t egid, gid_t sgid)
{
	su_class	su_event;

	// If no daemon, or the daemon has generated the event.
	if((!AUDIT_IS_RUNNING) || (AUDIT_IS_RUNNING && (current->pid == auditdaemon_task_struct->pid)))
	{
		return orig_setresgid32(rgid,egid,sgid);
	}

	// Set the current id data from the pre-system call user data
	su_event.t_header.user_id=current->uid;
	su_event.t_header.group_id=current->gid;
	su_event.t_header.euser_id=current->euid;
	su_event.t_header.egroup_id=current->egid;

	su_event.t_return.returncode=orig_setresgid32(rgid,egid,sgid);

	su_event.t_header.event_class=AUDIT_CLASS_SU;
	su_event.t_header.event_id=SYS_setresgid32;
	su_event.t_process.pid=current->pid;
	su_event.t_process.name[0]='\0';
	strncpy(su_event.t_process.name,current->comm,MAXCOMMAND);

	su_event.t_target.id=egid;
	su_event.t_target.rid=rgid;
	su_event.t_target.sid=sgid;

	// Everything else can be handled by the generic event handler.
	audit_event(&su_event,1);

	// Now continue on with the normal exit.
	return su_event.t_return.returncode;
}


// audit file truncates.
// NOTE: This event is never called! Linux works out the filename, and calls truncate() instead!
/*asmlinkage int	audit_ftruncate (int fd, off_t length)
{
	io_class	io_event;

	// If no daemon, or the daemon has generated the event.
	if((!AUDIT_IS_RUNNING) || (AUDIT_IS_RUNNING && (current->pid == auditdaemon_task_struct->pid)))
	{
		return orig_ftruncate(fd,length);
	}

	io_event.t_return.returncode=orig_ftruncate(fd,length);

	io_event.t_header.event_class=AUDIT_CLASS_IO;
	io_event.t_header.event_id=SYS_ftruncate;
	io_event.t_process.pid=current->pid;
	io_event.t_process.name[0]='\0';
	strncpy(io_event.t_process.name,current->comm,MAXCOMMAND);

	// Ftruncate has no path, just a file int.
	// zero the path, just in case strncpy fails.
	io_event.t_path.path[0]='\0';
	// Would prefer to use snprintf here, but it's not available within the module
	// As long as MAX_PATH has enough storage for the text representation of an
	// integer, we'll be fine (ie: no buffer overflow).
	sprintf(io_event.t_path.path,"FD=%d",fd);
	
	// Save off the offset.
	io_event.t_attributes.createmode=(unsigned long)length;

	// Everything else can be handled by the generic event handler.
	audit_event(&io_event,0);

	// Now continue on with the normal exit.
	return io_event.t_return.returncode;
}*/

// NOTE: This event is never called! Linux works out the filename, and calls chmod() instead!
/*asmlinkage int audit_fchmod (int filedes, mode_t mode)
{
	io_class	io_event;

	io_event.t_return.returncode=orig_fchmod(filedes, mode);

	// If no daemon, or the daemon has generated the event.
	if((!AUDIT_IS_RUNNING) || (AUDIT_IS_RUNNING && (current->pid == auditdaemon_task_struct->pid)))
	{
		return(io_event.t_return.returncode);
	}


	io_event.t_header.event_class=AUDIT_CLASS_IO;
	io_event.t_header.event_id=SYS_chmod;

	// Process data
	io_event.t_process.pid=current->pid;
	io_event.t_process.name[0]='\0';
	strncpy(io_event.t_process.name,current->comm,MAXCOMMAND);

	io_event.t_path.path[0]='\0';
	sprintf(io_event.t_path.path,"FD=%d",filedes);

	io_event.t_attributes.mode=(int)NULL;
	io_event.t_attributes.createmode=(unsigned long)mode;

	// Everything else can be handled by the generic event handler.
	audit_event(&io_event,0);

	return(io_event.t_return.returncode);
}*/


// audit file truncates.
// NOTE: This event is never called! Linux works out the filename, and calls truncate64() instead!
/*asmlinkage int	audit_ftruncate64 (int fd, off_t length)
{
	io_class	io_event;

	// If no daemon, or the daemon has generated the event.
	if((!AUDIT_IS_RUNNING) || (AUDIT_IS_RUNNING && (current->pid == auditdaemon_task_struct->pid)))
	{
		return orig_ftruncate64(fd,length);
	}

	io_event.t_return.returncode=orig_ftruncate64(fd,length);

	io_event.t_header.event_class=AUDIT_CLASS_IO;
	io_event.t_header.event_id=SYS_ftruncate64;
	io_event.t_process.pid=current->pid;
	io_event.t_process.name[0]='\0';
	strncpy(io_event.t_process.name,current->comm,MAXCOMMAND);

	// Ftruncate has no path, just a file int.
	// zero the path, just in case strncpy fails.
	io_event.t_path.path[0]='\0';
	sprintf(io_event.t_path.path,"FD=%d",fd);
	
	// Save off the offset.
	io_event.t_attributes.createmode=(unsigned long)length;

	// Everything else can be handled by the generic event handler.
	audit_event(&io_event,0);

	// Now continue on with the normal exit.
	return io_event.t_return.returncode;
}*/

// audit file truncates.
asmlinkage int	audit_truncate64 (const char *path, off_t length)
{
	io_class	io_event;

	// If no daemon, or the daemon has generated the event.
	if((!AUDIT_IS_RUNNING) || (AUDIT_IS_RUNNING && (current->pid == auditdaemon_task_struct->pid)))
	{
		return orig_truncate64(path,length);
	}

	io_event.t_return.returncode=orig_truncate64(path,length);

	io_event.t_header.event_class=AUDIT_CLASS_IO;
	io_event.t_header.event_id=SYS_truncate64;
	io_event.t_process.pid=current->pid;
	io_event.t_process.name[0]='\0';
	strncpy(io_event.t_process.name,current->comm,MAXCOMMAND);

	// Store the path
	// zero the path, just in case strncpy fails.
	io_event.t_path.path[0]='\0';
	if(path) {
		strncpy_from_user(io_event.t_path.path,path,MAX_PATH);
	}
	
	// Save off the offset.
	io_event.t_attributes.createmode=(unsigned long)length;

	// Everything else can be handled by the generic event handler.
	audit_event(&io_event,0);

	// Now continue on with the normal exit.
	return io_event.t_return.returncode;
}

// fchown is not actually called! Linux calls chown instead (chown32)!
/*asmlinkage int audit_fchown(int fd, uid_t owner, gid_t group)
{
	// Dunno whether we should use an io class for chown.
	ch_class	ch_event;

	// If no daemon, or the daemon has generated the event.
	if((!AUDIT_IS_RUNNING) || (AUDIT_IS_RUNNING && (current->pid == auditdaemon_task_struct->pid)))
	{
		return orig_fchown(fd,owner,group);
	}

	ch_event.t_return.returncode=orig_fchown(fd,owner,group);

	ch_event.t_header.event_class=AUDIT_CLASS_CH;
	ch_event.t_header.event_id=SYS_fchown;
	ch_event.t_process.pid=current->pid;
	ch_event.t_process.name[0]='\0';
	strncpy(ch_event.t_process.name,current->comm,MAXCOMMAND);

	ch_event.t_owner.owner=(int) owner;
	ch_event.t_owner.group=(int) group;

	// Fchown has no path, just a file int.
	// zero the path, just in case strncpy fails.
	ch_event.t_path.path[0]='\0';
	sprintf(ch_event.t_path.path,"FD=%d",fd);

	// Everything else can be handled by the generic event handler.
	audit_event(&ch_event,0);

	// Now continue on with the normal exit.
	return ch_event.t_return.returncode;

}*/

// Change this so we have an idea of what sort of socket is being opened.
// NOTE: report in the format 10.0.0.2:1234 (so match can look at ip and/or port!)
//
asmlinkage int audit_socketcall(int call, unsigned long *args)
{
	nt_class	nt_event;

	// If no daemon, or the daemon has generated the event.
	if((!AUDIT_IS_RUNNING) || (AUDIT_IS_RUNNING && (current->pid == auditdaemon_task_struct->pid)) ||
	(call != SYS_CONNECT && call != SYS_ACCEPT))
	{
		return(orig_socketcall(call,args));
	}

	nt_event.t_return.returncode=orig_socketcall(call, args);

	nt_event.t_header.event_class=AUDIT_CLASS_NET;
	nt_event.t_header.event_id=SYS_socketcall;

	// Process data
	nt_event.t_process.pid=current->pid;
	nt_event.t_process.name[0]='\0';
	strncpy(nt_event.t_process.name,current->comm,MAXCOMMAND);

	if(call == SYS_CONNECT) {
		struct sockaddr *auditsockaddr;
		struct sockaddr_in *audit_in;
			
		auditsockaddr=(struct sockaddr *)args[1];

		nt_event.syscall=SYS_CONNECT;
		
		if(auditsockaddr->sa_family == AF_INET) {
			char *p;
			audit_in=(struct sockaddr_in *)auditsockaddr;
			p=(char*)&audit_in->sin_addr.s_addr;
			
			// Work out whether the ip address (including dots) would fit in our buffer
			if((unsigned char)*(p+0) <=255 && (unsigned char)*(p+1) <=255 &&  (unsigned char)*(p+2) <= 255 && (unsigned char)*(p+3) <= 255) {
				// Would REALLY prefer snprintf here.
				sprintf(nt_event.t_connection.dst_ip,"%d.%d.%d.%d",(unsigned char)*(p+0),(unsigned char)*(p+1),(unsigned char)*(p+2),(unsigned char)*(p+3));
			} else {
				sprintf(nt_event.t_connection.dst_ip,"Unknown");
			}
			strcpy(nt_event.t_connection.src_ip,"127.0.0.1");
			nt_event.t_connection.dst_port=ntohs(audit_in->sin_port);
			nt_event.t_connection.src_port=0; // Unknown
			
		} else {
			// Not inet? Not interested.
			return(nt_event.t_return.returncode);
		}			
	} else if(call == SYS_ACCEPT) {
		struct sockaddr *auditsockaddr;
		struct sockaddr_in *audit_in;
		
		auditsockaddr=(struct sockaddr *)args[1];
			
		nt_event.syscall=SYS_ACCEPT;
		if(auditsockaddr->sa_family == AF_INET) {
			char *p;
			audit_in=(struct sockaddr_in *)auditsockaddr;
			p=(char*)&audit_in->sin_addr.s_addr;

			// Work out whether the ip address (including dots) would fit in our buffer
			if((unsigned char)*(p+0) <=255 && (unsigned char)*(p+1) <=255 &&  (unsigned char)*(p+2) <= 255 && (unsigned char)*(p+3) <= 255) {
				// Would REALLY prefer snprintf here.
				sprintf(nt_event.t_connection.src_ip,"%d.%d.%d.%d",(unsigned char)*(p+0),(unsigned char)*(p+1),(unsigned char)*(p+2),(unsigned char)*(p+3));
			} else {
				sprintf(nt_event.t_connection.src_ip,"Unknown");
			}
			nt_event.t_connection.src_port=ntohs(audit_in->sin_port);
			
			strcpy(nt_event.t_connection.dst_ip,"127.0.0.1");
			nt_event.t_connection.dst_port=0; // Unknown. I really would like to know though, since this is the port on the local machine that they are connecting to!
		} else {
			// Not inet? Not interested.
			return(nt_event.t_return.returncode);
		}			

	}

	// Everything else can be handled by the generic event handler.
	audit_event(&nt_event,0);

	return(nt_event.t_return.returncode);
}

// Change this so we can grab the name. Will use a different class.
asmlinkage caddr_t audit_create_module(const char *name, size_t size)
{
	ad_class	ad_event;
	caddr_t		returncode;
	
	// If no daemon, or the daemon has generated the event.
	if((!AUDIT_IS_RUNNING) || (AUDIT_IS_RUNNING && (current->pid == auditdaemon_task_struct->pid)))
	{
		return(orig_create_module(name,size));
	}
	
	returncode=orig_create_module(name,size);
	
	ad_event.t_return.returncode=(int)returncode;

	ad_event.t_header.event_class=AUDIT_CLASS_AD;
	ad_event.t_header.event_id=SYS_create_module;

	// Process data
	ad_event.t_process.pid=current->pid;
	ad_event.t_process.name[0]='\0';
	strncpy(ad_event.t_process.name,current->comm,MAXCOMMAND);

	ad_event.t_name.path[0]='\0';
	if(name) {
		strncpy_from_user(ad_event.t_name.path,name,MAX_PATH);
	}

	// Everything else can be handled by the generic event handler.
	audit_event(&ad_event,0);

	return(returncode);
}

// Change this so we can grab the name. Will use a different class.
// NOTE: Have removed the capability to audit delete_module, since auditmodule will segfault if it tries to remove itself!
/*asmlinkage caddr_t audit_delete_module(const char *name)
{
	ad_class	ad_event;
	caddr_t		returncode;

	// If no daemon, or the daemon has generated the event.
	if((!AUDIT_IS_RUNNING) || (AUDIT_IS_RUNNING && (current->pid == auditdaemon_task_struct->pid)))
	{
		return(orig_delete_module(name));
	}

	returncode=orig_delete_module(name);
	ad_event.t_return.returncode=(int)returncode;

	ad_event.t_header.event_class=AUDIT_CLASS_AD;
	ad_event.t_header.event_id=SYS_delete_module;

	// Process data
	ad_event.t_process.pid=current->pid;
	ad_event.t_process.name[0]='\0';
	strncpy(ad_event.t_process.name,current->comm,MAXCOMMAND);

	ad_event.t_name.path[0]='\0';
	if(name) {
		strncpy_from_user(ad_event.t_name.path,name,MAX_PATH);
	}

	// Everything else can be handled by the generic event handler.
	audit_event(&ad_event,0);

	return(returncode);
}*/

